// ===========================================================================
// CONTENT  : INTERFACE IType
// AUTHOR   : M.Duchrow
// VERSION  : 1.1 - 16/08/2007
// HISTORY  :
//  07/01/2007  mdu  CREATED
//	16/08/2007	mdu		added		-->	isFinal(), isAbstract(), getVisibility()
//
// Copyright (c) 2007, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.odem;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.Collection;

/**
 * Represents a type such as a class, an interface, an enumeration and so on.
 *
 * @author M.Duchrow
 * @version 1.1
 */
public interface IType extends IExplorableElement
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  /**
   * An empty array of IType elements that can be used as return value
   * in implementations whenever an empty array is needed.
   */
  public static final IType[] EMPTY_TYPE_ARRAY = new IType[0];

  // =========================================================================
  // METHODS
  // =========================================================================
  /**
   * Returns the classification (i.e. CLASS, INTERFACE, ANNOTAION, ENUM) 
   * of this type.
   */
  public TypeClassification getClassification();

  /**
   * Returns the namespace the type belongs to.
   */
  public INamespace getNamespace();

  /**
   * Returns the name without the namespace
   */
  public String getUnqualifiedName();

  /**
   * Returns all types this type directly depends on.
   * Each type must be only once in the result collection.
   * This method never returns null.
   * <p/>
   * This is a convenience method that allows to work on the referred types
   * rather than the more detailed information provided by the result of
   * {@link #getDependencies()}.
   * 
   * @return A collection zero or many referred types without duplicates
   */
  public Collection<IType> getDirectReferredTypes();

  /**
   * Returns true if this type extends the given type directly.
   * <p/>
   * This is a convenient method to avoid the necessity of always iterating 
   * over the dependencies provided by {@link #getDependencies()} if one
   * wants to find out if there is a direct inheritance relationship.
   * 
   * @param type Any type that can be extended.
   * @see #isDirectImplementorOf(IType)
   * @see #getDependencies()
   * @return true if this type is a direct extension of the given type.
   */
  public boolean isDirectExtensionOf(IType type);

  /**
   * Returns true if this type directly implements the given type.
   * If the given type is not an interface false wiil be returned.
   * <p/>
   * This is a convenient method to avoid the necessity of always iterating 
   * over the dependencies provided by {@link #getDependencies()} if one
   * wants to find out if there is a direct implementation relationship.
   * 
   * @param type An interface type
   * @see #isDirectExtensionOf(IType)
   * @see #getDependencies()
   * @return true if this type is a direct implementor of the given interface type.
   */
  public boolean isDirectImplementorOf(IType type);

  /**
   * Returns all dependencies to other types. That include all different dependencies
   * to the same type (fields, method signatures, inheritance etc.).
   * <br/>
   * To get just the different types without duplicates this type depends on 
   * use {@link #getDirectReferredTypes()}.
   */
  public DependencySet<IType, IType> getDependencies();

  /**
   * Returns whether or not this type is final.
   */
  public boolean isFinal();

  /**
   * Returns whether or not this type is abstract.
   */
  public boolean isAbstract();

  /**
   * Returns the visibility of this type.
   * Or course only the visibilities {@link Visibility#PUBLIC} and 
   * {@link Visibility#DEFAULT} are reasonable return values here. 
   */
  public Visibility getVisibility();
}