// ===========================================================================
// CONTENT  : CLASS ObjectProperties
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 12/01/2007
// HISTORY  :
//  12/01/2007  mdu  CREATED
//
// Copyright (c) 2007, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.odem;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.HashMap;
import java.util.Map;

/**
 * This is an implementation of IObjectProperties which is provided here
 * as convenience, so that not everybody is forced create his own implementation.
 *
 * @author M.Duchrow
 * @version 1.0
 */
public class ObjectProperties implements IObjectProperties
{
  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private Map<String, Object> properties = new HashMap<String, Object>();

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public ObjectProperties()
  {
    super();
  }

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Returns the property associated with the given name. A null value
   * indicates that a property with the specified name does not exist.
   * 
   * @param propertyName A valid name (must not be null or empty string)
   * @return The object associated with the given name or null if property unknown
   */
  @Override
  public Object getProperty(String propertyName)
  {
    if (!this.isNullOrEmpty(propertyName))
    {
      return properties.get(propertyName);
    }
    return null;
  }

  /**
   * Sets the value of the property with the specified name to the given object.
   * This replaces ny object that might have been kept under the given name before.
   * <p/>
   * If the given object is null, the property and any associated value will be
   * removed.
   * 
   * @param propertyName A valid name (must not be null or empty string)
   * @param object The object to keep under the specified property name
   */
  @Override
  public void setProperty(String propertyName, Object object)
  {
    if (!this.isNullOrEmpty(propertyName))
    {
      if (object == null)
      {
        properties.remove(propertyName);
      }
      else
      {
        properties.put(propertyName, object);
      }
    }
  }

  /**
   * Returns all currently known properties. 
   * If no property is known an empty array will be returned (never null).
   * 
   * @return All the names of available properties 
   */
  @Override
  public String[] getPropertyNames()
  {
    String[] names;

    names = new String[properties.size()];
    properties.keySet().toArray(names);
    return names;
  }

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected boolean isNullOrEmpty(String str)
  {
    return (str == null) || (str.length() == 0);
  }
}
